/*
  teem: Gordon Kindlmann's research software
  Copyright (C) 2002, 2001, 2000, 1999, 1998 University of Utah

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "air.h"

/*
******** airSanity()
**
** Does run-time checks to see if the compile-time constants are correct.
** Returns a value 
*/
int
airSanity(void) {
  double nan, pinf, ninf;
  float nanF, pinfF, ninfF;
  int tmpI, sign, exp, frac;
  char endian;
  static int sanity=0;
  
  if (sanity) {
    return airInsane_not;
  }

  /* run-time endian check */
  tmpI = 1;
  endian = !(*((char*)(&tmpI)));
  if (endian) {
    /* big endian */
    if (4321 != airMyEndian) {
      return airInsane_endian;
    }
  }
  else {
    if (1234 != airMyEndian) {
      return airInsane_endian;
    }
  }    

  /* run-time NaN checks */
  pinf = 1.7e+300;                  /* close to DBL_MAX */
  pinf = pinf * pinf * pinf * pinf;
  pinf = pinf * pinf * pinf * pinf;
  pinf = pinf * pinf * pinf * pinf;
  pinf = pinf * pinf * pinf * pinf;
  if (AIR_EXISTS(pinf)) {
    return airInsane_pInfExists;
  }
  ninf = -pinf;
  if (AIR_EXISTS(ninf)) {
    return airInsane_nInfExists;
  }
  nan = pinf/pinf;
  if (AIR_EXISTS(nan)) {
    return airInsane_NaNExists;
  }
  nanF = nan;
  pinfF = pinf;
  ninfF = ninf;
  if (!(airFP_QNAN == airFPClass_f(nanF)
	&& airFP_POS_INF == airFPClass_f(pinfF)
	&& airFP_NEG_INF == airFPClass_f(ninfF))) {
    /* really, this is verifying that assigning from a double to a 
       float maintains the FPClass for non-existant values */
    return airInsane_FltDblFPClass;
  }
  airFPValToParts_f(&sign, &exp, &frac, nanF);
  frac >>= 22;
  if (airMyQNaNHiBit != frac) {
    return airInsane_QNaNHiBit;
  }
  
  /* just make sure airMyDio is reasonably set */
  switch (airMyDio) {
  case 0: break;
  case 1: break;
  default:
    return airInsane_dio;
  }

  sanity = 1;
  return airInsane_not;
}

const char
_airInsaneErr[9][AIR_STRLEN_MED] = {
  "(invalid insane value)",
  "sanity checked PASSED!",
  "airMyEndian is wrong",
  "AIR_EXISTS(+inf) was true",
  "AIR_EXISTS(-inf) was true",
  "AIR_EXISTS(NaN) was true",
  "air_FPClass_f() wrong after double->float assignment",
  "airMyQNaNHiBit is wrong",
  "airMyDio has invalid value"
};

const char *
airInsaneErr(int insane) {
  
  if (AIR_INSIDE(0, insane, 7)) {
    return _airInsaneErr[insane+1];
  }
  else {
    return _airInsaneErr[0];
  }
}

